//+------------------------------------------------------------------+
//|                                        ForexFactoryNewsTable.mq4 |
//|                        Copyright 2021, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict
#include "WebRequests_2021.mqh";
enum impacts
{
impact_low=1,//low and above
impact_medium=2,//medium and above
impact_high=3//high
};
input int ManualTimeZoneMinutesOffset=-240;//Metatraders request offset in minutes from utc
input bool AutoTimeZone=true;//auto find the above 
input uint RequestTimeout=2000;//request timeout milliseconds
input int TimeForAlert=15;//minutes to alert before event
input impacts AlertTheseImpacts=impact_high;//Impact of news for preparation alert
input bool alertPopup=true;//Popup alert
input bool alertMail=false;//Email alert
input bool alertPush=false;//Phone alert
input string notea="<->";//DISPLAY SETTINGS
input int T_Rows=10;//Rows to show 
input int T_TableRowHeight=20;//Row height
input string T_TableFont="Arial";//Font:
input int T_TableFontSize=12;//Font Size
input color T_NavigatorBorderColor=clrDodgerBlue;//Navigator border color
input color T_NavigatorBackColor=clrMidnightBlue;//Navigator back color
input color T_NavigatorTextColor=clrOrange;//Navigator text color
input color T_ButtonBorderColor=clrGainsboro;//Buttons border color
input color T_ButtonBackColor=clrBlack;//Buttons back color
input color T_ButtonTextColor=clrWhite;//Buttons text color
input color T_HeaderBorderColor=clrBlue;//Header border color
input color T_HeaderBackColor=clrDarkBlue;//Header back color
input color T_HeaderTextColor=clrWhiteSmoke;//Header text color
input color T_RowAColor=clrGray;//Data row 1 color
input color T_RowATextColor=clrBlack;//Data row 1 text color
input color T_RowBColor=clrDarkGray;//Data row 2 color
input color T_RowBTextColor=clrBlack;//Data row 2 text color
input color T_ScrollBackColor=clrGray;//Scroll Back color
input color T_SettingsBackColor=clrBlack;//Settings Back Color
input color T_SettingsBorderColor=clrWhite;//Settings Border Color
input color T_SettingsRowAColor=clrGainsboro;//Settings Row A Color
input color T_SettingsRowBColor=clrWhite;//Settings Row B Color
input color T_SettingsTextColor=clrBlack;//Settings Text Color
input color T_SettingsCheckMarkColor=clrYellowGreen;//Settings check mark color
input color T_SettingsCheckMarkBackColor=clrBlack;//Settings check mark back color
input color T_ColorGrade_None=clrBlack;//Empty Color Grade
input color T_ColorGrade_Insignificant=clrGray;//Insignificant Color Grade
input color T_ColorGrade_Low=clrGreen;//Low Color Grade
input color T_ColorGrade_Medium=clrOrange;//Medium Color Grade
input color T_ColorGrade_High=clrRed;//High Color Grade
string the_scroll_font="Webdings";
int the_scroll_font_size=0;
//THE CACHE IS ON ,DONT FORGET TO TURN IT OFF!
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
string SystemTag="FFNT_";
bool DeckReady=false;
/*
so ,we will have the current day which will manage the news of the day and timers for receiving updates, that will always 
be online.
and we have the navigation day ,this will allow user to navigate in days while the alert arm stays online.
the navigated day can be also loaded if the chart has not changed
*/
string SystemFolder="FFNews",NavSubFolder="Navs",CacheFolder="Cache";
struct nav_data
{
//prev now next for the display part
datetime display_prev,display_now,display_next;
//prev now next for the alert part
datetime alert_prev,alert_now,alert_next;
//last alerted time 
datetime last_alerted;
//last actual time (that we loaded actual prints live)
datetime last_actual;
nav_data(void){display_prev=0;display_now=0;display_next=0;alert_prev=0;alert_now=0;alert_next=0;last_alerted=0;last_actual=0;}
};
nav_data NAV;
bool SettingsUp=false,HasTimer=false,SystemBusy=false;

int OnInit()
  {
//--- create timer 
  //setup ic
  setup_color_grades();
  DeckReady=false;SettingsUp=false;SystemBusy=true;
  ObjectsDeleteAll(ChartID(),SystemTag);
  the_scroll_font_size=T_TableFontSize;
  FFS.reset();
  //load ffs if it exists 
  load_ffs(SystemFolder,NavSubFolder,ChartID(),FFS);
  //check if navigation data exists for this chart 
  load_navigation_data(SystemFolder,NavSubFolder,ChartID(),NAV);
  //load news 
  bool load_monitor_news=getTripleData(NAV.alert_now,MonitorNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
  //if they are the same day as the display news 
  if(TimeDay(NAV.display_now)==TimeDay(NAV.alert_now)){
  if(ArraySize(MonitorNews)>0){ArrayResize(DisplayNews,ArraySize(MonitorNews),0);}
  else if(ArraySize(MonitorNews)==0){ArrayFree(DisplayNews);}
  for(int s=0;s<ArraySize(MonitorNews);s++){
  DisplayNews[s]=MonitorNews[s];
  }
  }else if(TimeDay(NAV.display_now)!=TimeDay(NAV.alert_now)){
  bool load_display_news=getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
  }
  NewsFiltration(FilteredNews,DisplayNews,FFS);
  premcc=NULL;premcx=0;premcy=0;
  build_ff_table(SystemTag,10,10,T_TableFont,T_TableFontSize,the_scroll_font,the_scroll_font_size,T_TableRowHeight,T_Rows);
  update_ff_table(SystemTag);
  ChartSetInteger(ChartID(),CHART_EVENT_MOUSE_MOVE,true);
  DeckReady=true;
  HasTimer=EventSetTimer(1);
  if(HasTimer){SystemBusy=false;}
  if(!HasTimer){ExpertRemove();Alert("Cannot start timer");}
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
  DeckReady=false;
  ObjectsDeleteAll(ChartID(),SystemTag);
  ArrayFree(DisplayNews);
  ArrayFree(MonitorNews);
  ArrayFree(FilteredNews);
  if(HasTimer){EventKillTimer();}
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
  //if not busy
    if(SystemBusy==false)
    {
    SystemBusy=true;
    bool navdataneedssave=false,needtopull=false,anything_informed=false;//need to pull refers to quickpolling
    //one page of news (since news at same times will be on the same page as well and 
    //we maintain a reference of which page that is
    string page_for_pull=NULL;//per the above
    datetime day_to_request=0;
    //check if its a new day and we need to update the monitor news 
    //and also , if the navigation is on the current day ,to move it forward
      if(TimeDay(TimeLocal())!=TimeDay(NAV.alert_now))
      {
      bool in_current_view=false;
      if(TimeDayOfYear(NAV.display_now)==TimeDayOfYear(NAV.alert_now)){in_current_view=true;}
      //do the loading stuff
        //load news 
          NAV.alert_now=TimeLocal();
          NAV.alert_prev=(datetime)(NAV.alert_now-86400);
          NAV.alert_next=(datetime)(NAV.alert_now+86400);        
        bool load_monitor_news=getTripleData(NAV.alert_now,MonitorNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
        //if they are the same day as the display news 
        if(in_current_view){
        if(ArraySize(MonitorNews)>0){ArrayResize(DisplayNews,ArraySize(MonitorNews),0);}
        else if(ArraySize(MonitorNews)==0){ArrayFree(DisplayNews);}
        for(int s=0;s<ArraySize(MonitorNews);s++){
        DisplayNews[s]=MonitorNews[s];
        }
        NewsFiltration(FilteredNews,DisplayNews,FFS);
        update_ff_table(SystemTag);}
        navdataneedssave=true;
      }
    //check if any news are coming in the following X minutes
    //we freely check the monitor news ,this array only fills up on load and on new days
      datetime last_alert_projection=(datetime)(TimeLocal()+TimeForAlert*60);
     string pair1=SymbolInfoString(_Symbol,SYMBOL_CURRENCY_MARGIN);
     string pair2=SymbolInfoString(_Symbol,SYMBOL_CURRENCY_PROFIT);
     for(int n=0;n<ArraySize(MonitorNews);n++)
      {
      //if we discover a news item upcoming 
        if(MonitorNews[n].prints&&MonitorNews[n].expecting&&NAV.last_alerted<MonitorNews[n].timelocal&&last_alert_projection>=MonitorNews[n].timelocal){
        //and it fits our criteria for alerts
        if(MonitorNews[n].impact>=(int)AlertTheseImpacts){
        //if its of this pair 
        if(pair1!=pair2)
        {
        string le_p1=MonitorNews[n].currency;
        if(StringFind(le_p1,pair1,0)!=-1||StringFind(le_p1,pair2,0)!=-1){    
        {
        inform_upcoming(MonitorNews[n]);
        anything_informed=true;
        }}}}}
      //if we discover a news item pending release 
        if(!needtopull)
        {
        if(MonitorNews[n].prints&&MonitorNews[n].expecting&&TimeLocal()>=MonitorNews[n].timelocal){
        needtopull=true;
        page_for_pull=MonitorNews[n].request_param_to_get_this;
        day_to_request=MonitorNews[n].datetime_requested;
        navdataneedssave=true;
        }}
      }
      if(anything_informed){NAV.last_alerted=last_alert_projection;navdataneedssave=true;}
    //if need to pull a singleton
      if(needtopull)
      {
      bool singlepull=requestOneWithParameter(page_for_pull,day_to_request);
      bool in_current_view=false;
      if(TimeDayOfYear(NAV.display_now)==TimeDayOfYear(NAV.alert_now)){in_current_view=true;}
        //if they are the same day as the display news 
        if(in_current_view){
        if(ArraySize(MonitorNews)>0){ArrayResize(DisplayNews,ArraySize(MonitorNews),0);}
        else if(ArraySize(MonitorNews)==0){ArrayFree(DisplayNews);}
        for(int s=0;s<ArraySize(MonitorNews);s++){
        DisplayNews[s]=MonitorNews[s];
        }
        NewsFiltration(FilteredNews,DisplayNews,FFS);
        update_ff_table(SystemTag);}      
      }
    //if need to save navs
      if(navdataneedssave)
      {
      save_navigation_data(SystemFolder,NavSubFolder,ChartID(),NAV);
      }
      
    SystemBusy=false;
    }
  //if not busy  
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
int mcx,mcy,premcx,premcy;
string mcc,premcc;
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
  //if system not busy
  if(SystemBusy==false)
  {
  
  //if deck is ready 
  if(DeckReady&&!SettingsUp)
  {
  
  //mousemove 
  if(id==CHARTEVENT_MOUSE_MOVE){
  mcx=(int)lparam;
  mcy=(int)dparam;
  mcc=sparam;
  //scrolling 
    if(scrolling){
    scroll_offset=scrolliginal+(mcy-scrolltether);
    if(mcc=="0"){
    scrolling=false;
    if(scrollmemory){ChartSetInteger(ChartID(),CHART_MOUSE_SCROLL,true);}
    }
    update_ff_table(SystemTag);
    }
  //not scrolling 
    else if(!scrolling){
    int se_x=scroll_bar_px+T_TableRowHeight;
    int se_y=scroll_bar_py+scroll_bar_size;
    //if was inside with no click ,is inside and click enable scrolling 
      if(premcx>=scroll_bar_px&&premcx<=se_x&&premcy>=scroll_bar_py&&premcy<=se_y&&premcc=="0"){
        if(mcx>=scroll_bar_px&&mcx<=se_x&&mcy>=scroll_bar_py&&mcy<=se_y&&mcc=="1"){
        scrollmemory=(bool)(ChartGetInteger(ChartID(),CHART_MOUSE_SCROLL));
        scrolling=true;
        ChartSetInteger(ChartID(),CHART_MOUSE_SCROLL,false);
        scrolltether=mcy;
        scrolliginal=scroll_offset;
      }}
    }
  premcx=mcx;
  premcy=mcy;
  premcc=mcc;
  }
  
  if(!scrolling){
  //scroll up
    if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_ScrollUP"){
    scroll_offset--;
    update_ff_table(SystemTag);
    }
  //scroll down
    else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_ScrollDOWN"){
    scroll_offset++;
    update_ff_table(SystemTag);
    }
  //previous navigation
    else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_Prev_Navigation")
    {
    DeckReady=false;
    NAV.display_now=NAV.display_prev;
    NAV.display_prev=(datetime)(NAV.display_now-86400);
    NAV.display_next=(datetime)(NAV.display_now+86400);
    scroll_offset=0;
    ArrayFree(DisplayNews);
    if(NAV.display_now<NAV.alert_now)
    {
    getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
    }
    else
    {getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,false,true);}
    
    NewsFiltration(FilteredNews,DisplayNews,FFS);
    update_ff_table(SystemTag);
    DeckReady=true;
    save_navigation_data(SystemFolder,NavSubFolder,ChartID(),NAV); 
    }
  //next navigation
    else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_Next_Navigation")
    {
    DeckReady=false;
    NAV.display_now=NAV.display_next;
    NAV.display_prev=(datetime)(NAV.display_now-86400);
    NAV.display_next=(datetime)(NAV.display_now+86400);
    scroll_offset=0;
    ArrayFree(DisplayNews);
    if(NAV.display_now<NAV.alert_now)
    {
    getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
    }
    else
    {getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,false,true);}
    NewsFiltration(FilteredNews,DisplayNews,FFS);
    update_ff_table(SystemTag);
    DeckReady=true;   
    save_navigation_data(SystemFolder,NavSubFolder,ChartID(),NAV); 
    }  
  //now navigation
    else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_Now_Navigation")
    {
    DeckReady=false;
    NAV.display_now=TimeLocal();
    NAV.display_prev=(datetime)(NAV.display_now-86400);
    NAV.display_next=(datetime)(NAV.display_now+86400);
    scroll_offset=0;
    ArrayFree(DisplayNews);
    if(NAV.display_now<NAV.alert_now)
    {
    getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,true,true);
    }
    else
    {getTripleData(NAV.display_now,DisplayNews,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,false,true);}
    NewsFiltration(FilteredNews,DisplayNews,FFS);
    update_ff_table(SystemTag);
    DeckReady=true;   
    save_navigation_data(SystemFolder,NavSubFolder,ChartID(),NAV); 
    }      
  //settings 
    else if(id==CHARTEVENT_OBJECT_CLICK&&(sparam==SystemTag+"_Settings"||sparam==SystemTag+"_Settings2")){
    build_settings_table(setts_px,setts_py,T_TableRowHeight,SystemTag,T_SettingsBackColor,T_SettingsBorderColor,T_SettingsRowAColor,T_SettingsRowBColor,T_SettingsTextColor,T_SettingsCheckMarkColor,T_SettingsCheckMarkBackColor,IC,FFS);
    }
  }

  }
  //if Deckready ends here
  //settings
  else if(DeckReady&&SettingsUp){
  if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_INSIGNIFICANT"){
  FFS.show_insignificant=!FFS.show_insignificant;
  update_settings(SystemTag,FFS);  
  }  
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_LOW"){
  FFS.show_low_impact=!FFS.show_low_impact;
  update_settings(SystemTag,FFS);  
  } 
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_MEDIUM"){
  FFS.show_medium_impact=!FFS.show_medium_impact;
  update_settings(SystemTag,FFS);  
  } 
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_HIGH"){
  FFS.show_high_impact=!FFS.show_high_impact;
  update_settings(SystemTag,FFS);  
  }   
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_ALL"){
  FFS.show_only_relative=false;
  update_settings(SystemTag,FFS);  
  } 
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_SELECTOR_PAIR"){
  FFS.show_only_relative=true;
  update_settings(SystemTag,FFS);  
  }   
  else if(id==CHARTEVENT_OBJECT_CLICK&&sparam==SystemTag+"_SET_OKAY"){
  ObjectsDeleteAll(ChartID(),SystemTag+"_SET_");
  save_ffs(SystemFolder,NavSubFolder,ChartID(),FFS);
  NewsFiltration(FilteredNews,DisplayNews,FFS);
  update_ff_table(SystemTag);
  SettingsUp=false;
  } 
  }
  
  }else if(SystemBusy){Alert("Please wait");}
  //if system not busy ends here
  }
//+------------------------------------------------------------------+

//design table :
bool scrolling=false,scrollmemory=false;
int scroll_available_y,scroll_bar_size,scroll_bar_px,scroll_bar_py,scroll_offset,scroll_top_y,scrolltether,scrolliginal;
int setts_px=0,setts_py=0;
void build_ff_table(string tag,
                    int px,int py,
                    string font,
                    int fontsize,
                    string scroll_font,
                    int scroll_fontsize,
                    int rowheight,
                    int total_rows){
int drawx=px;
int drawy=px;
//find max size needed 
string rowtitles[]={"Time","C","I","Title","Actual","Forecast","Previous"};
string testitles[]={"00:00","DUSD","33","Dear All, I am facing a problem with my","3333","3333","3333"};
int    sizes[];
ArrayResize(sizes,ArraySize(rowtitles),0);
int tally=0;
TextSetFont(font,fontsize,0,0);
for(int t=0;t<ArraySize(rowtitles);t++){
uint h,w,th,tw;
TextGetSize(rowtitles[t],w,h);
TextGetSize(testitles[t],tw,th);
sizes[t]=(int)w+(int)(((double)w)*1.3);
if(tw>w){sizes[t]=(int)tw+(int)(((double)tw)*1.3);}
tally+=sizes[t];
}
string objna="";
int row_size_x=tally;
//background header
  objna=tag+"_Navigation_Background";
  HS_Create_Btn(ChartID(),0,objna,row_size_x,rowheight,drawx,drawy,font,fontsize,T_NavigatorBackColor,T_NavigatorBorderColor,BORDER_FLAT,T_NavigatorTextColor,ALIGN_CENTER,"",false,false);
//previous date button 
  objna=tag+"_Prev_Navigation";
  HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,scroll_font,scroll_fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"3",false,false);
//which date in display 
  int displayx=row_size_x/2-rowheight*2;
  objna=tag+"_Navigation_Date_Display";
  drawx+=rowheight;
  HS_Create_Btn(ChartID(),0,objna,displayx,rowheight,drawx,drawy,font,fontsize,T_NavigatorBackColor,T_NavigatorBorderColor,BORDER_FLAT,T_NavigatorTextColor,ALIGN_CENTER,"",false,false);
//next date button
  drawx+=displayx;
  objna=tag+"_Next_Navigation";
  HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,scroll_font,scroll_fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"4",false,false);
//now date button 
  drawx+=rowheight*2;
  objna=tag+"_Now_Navigation";
  HS_Create_Btn(ChartID(),0,objna,rowheight*3,rowheight,drawx,drawy,font,fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"Now",false,false);
//settings button
  drawx=px+row_size_x-rowheight;
  objna=tag+"_Settings";
  setts_px=drawx;setts_py=drawy+rowheight;
  HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,font,fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"",false,false);
  objna=tag+"_Settings2";
  bool obji=ObjectCreate(ChartID(),objna,OBJ_LABEL,0,0,0);
  ObjectSetInteger(ChartID(),objna,OBJPROP_ANCHOR,ANCHOR_CENTER);
  ObjectSetInteger(ChartID(),objna,OBJPROP_ALIGN,ALIGN_CENTER);
  ObjectSetInteger(ChartID(),objna,OBJPROP_XDISTANCE,drawx+rowheight/2);
  ObjectSetInteger(ChartID(),objna,OBJPROP_YDISTANCE,drawy+rowheight/2+rowheight/4+rowheight/8);
  ObjectSetInteger(ChartID(),objna,OBJPROP_FONTSIZE,fontsize*2);
  ObjectSetString(ChartID(),objna,OBJPROP_FONT,font);
  ObjectSetInteger(ChartID(),objna,OBJPROP_COLOR,T_ButtonTextColor);
  ObjectSetString(ChartID(),objna,OBJPROP_TEXT,"*");
  ObjectSetInteger(ChartID(),objna,OBJPROP_SELECTABLE,false);
  ObjectSetInteger(ChartID(),objna,OBJPROP_BACK,false);
//header 
  drawx=px;
  drawy+=rowheight;
  //loop in titles 
  for(int t=0;t<ArraySize(rowtitles);t++){
  objna=tag+"_"+rowtitles[t]+"_header";
  HS_Create_Btn(ChartID(),0,objna,sizes[t],rowheight,drawx,drawy,font,fontsize,T_HeaderBackColor,T_HeaderBorderColor,BORDER_FLAT,T_HeaderTextColor,ALIGN_CENTER,rowtitles[t],false,false);
  drawx+=sizes[t];
  }  
  //loop in data rows and alternate colors 
  int colorselector=0;
  drawy+=rowheight;
  for(int r=0;r<total_rows;r++){
  colorselector++;
  if(colorselector>1){colorselector=0;}
  color bakolor=T_RowAColor,texcolor=T_RowATextColor;
  if(colorselector==1){bakolor=T_RowBColor;texcolor=T_RowBTextColor;}
  //loop to columns 
  drawx=px;
    for(int t=0;t<ArraySize(rowtitles);t++){
    objna=tag+"_"+rowtitles[t]+"_"+IntegerToString(r);
    HS_Create_Btn(ChartID(),0,objna,sizes[t],rowheight,drawx,drawy,font,fontsize,bakolor,bakolor,BORDER_FLAT,texcolor,ALIGN_CENTER," ",false,false);
    drawx+=sizes[t];
    }
  //loop to columns ends here 
  drawy+=rowheight;
  }
//scrollers 
  drawx=px+row_size_x;
  drawy=py+rowheight*2;
  //scroll up 
    objna=tag+"_ScrollUP";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,scroll_font,scroll_fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"5",false,false);
  //scroll back
    drawy+=rowheight;
    objna=tag+"_ScrollBack";
    int availy=(total_rows-2)*rowheight;
    scroll_available_y=availy;
    scroll_top_y=drawy;
    HS_Create_Btn(ChartID(),0,objna,rowheight,availy,drawx,drawy,font,fontsize,T_ScrollBackColor,T_ScrollBackColor,BORDER_FLAT,T_ScrollBackColor,ALIGN_CENTER,"",false,false);
  //scroll bar 
    scroll_bar_size=availy;
    scroll_bar_px=drawx;
    scroll_bar_py=drawy;
    objna=tag+"_Scroller";
    HS_Create_Btn(ChartID(),0,objna,rowheight,availy,drawx,drawy,font,fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"",false,false);
  //scroll down
    drawy+=availy;
    objna=tag+"_ScrollDOWN";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,scroll_font,scroll_fontsize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"6",false,false);
    scrolling=false;
}

//update table 
void update_ff_table(string tag)
{
//objna=tag+"_Navigation_Date_Display";
  string monts[]={"None","January","February","March","April","May","June","July","August","September","October","November","December"};
  string thedayz[]={"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
  string dayz=IntegerToString(TimeDay(NAV.display_now));
  if(StringLen(dayz)<2){dayz="0"+dayz;}
  string DayData=monts[TimeMonth(NAV.display_now)]+dayz+" "+thedayz[TimeDayOfWeek(NAV.display_now)];
  ObjectSetString(ChartID(),SystemTag+"_Navigation_Date_Display",OBJPROP_TEXT,DayData);
//check scroller size and scroll bar size 
  int max_scroll_offset=ArraySize(FilteredNews)-T_Rows;
  if(max_scroll_offset<0){max_scroll_offset=0;}
  scroll_bar_size=scroll_available_y-max_scroll_offset;
  if(scroll_offset>max_scroll_offset){scroll_offset=max_scroll_offset;}
  if(scroll_offset<0){scroll_offset=0;}
  ObjectSetInteger(ChartID(),tag+"_Scroller",OBJPROP_YDISTANCE,scroll_top_y+scroll_offset);
  ObjectSetInteger(ChartID(),tag+"_Scroller",OBJPROP_YSIZE,scroll_bar_size);
  scroll_bar_py=scroll_top_y+scroll_offset;
  color cc[]={clrWhite,clrGreen,clrOrange,clrRed,clrBlue};
  for(int c=0;c<ArraySize(cc);c++){
  cc[0]=IC.insignificant;
  cc[1]=IC.low;
  cc[2]=IC.medium;
  cc[3]=IC.high;
  cc[4]=IC.none;
  }
//loop to rows 
  for(int r=0;r<T_Rows;r++)
  {
  int t=r+scroll_offset;
  //if data exists 
    if(t>=0&&t<ArraySize(FilteredNews)&&ArraySize(FilteredNews)>0)
    {
    ObjectSetString(ChartID(),tag+"_Title_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].title);
    string ts_hours=IntegerToString(TimeHour(FilteredNews[t].timelocal));
    string ts_minutes=IntegerToString(TimeMinute(FilteredNews[t].timelocal));
    if(StringLen(ts_hours)<2){ts_hours="0"+ts_hours;}
    if(StringLen(ts_minutes)<2){ts_minutes="0"+ts_minutes;}
    string timestring=ts_hours+":"+ts_minutes;
    if(FilteredNews[t].timelocal!=0){ObjectSetString(ChartID(),tag+"_Time_"+IntegerToString(r),OBJPROP_TEXT,timestring);}
    else{ObjectSetString(ChartID(),tag+"_Time_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].timestring);}
    ObjectSetString(ChartID(),tag+"_C_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].currency);
    if(FilteredNews[t].impact>=0){ObjectSetInteger(ChartID(),tag+"_I_"+IntegerToString(r),OBJPROP_BGCOLOR,cc[FilteredNews[t].impact]);}
    else{ObjectSetInteger(ChartID(),tag+"_I_"+IntegerToString(r),OBJPROP_BGCOLOR,cc[0]);}
    ObjectSetString(ChartID(),tag+"_Actual_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].actual);
    ObjectSetString(ChartID(),tag+"_Forecast_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].forecast);
    ObjectSetString(ChartID(),tag+"_Previous_"+IntegerToString(r),OBJPROP_TEXT,FilteredNews[t].previous);
    }
    else{
    ObjectSetString(ChartID(),tag+"_Title_"+IntegerToString(r),OBJPROP_TEXT,"-");
    string timestring="-";
    ObjectSetString(ChartID(),tag+"_Time_"+IntegerToString(r),OBJPROP_TEXT,timestring);   
    ObjectSetString(ChartID(),tag+"_C_"+IntegerToString(r),OBJPROP_TEXT,"-");
    ObjectSetInteger(ChartID(),tag+"_I_"+IntegerToString(r),OBJPROP_BGCOLOR,clrBlack);
    ObjectSetString(ChartID(),tag+"_Actual_"+IntegerToString(r),OBJPROP_TEXT,"-");
    ObjectSetString(ChartID(),tag+"_Forecast_"+IntegerToString(r),OBJPROP_TEXT,"-");
    ObjectSetString(ChartID(),tag+"_Previous_"+IntegerToString(r),OBJPROP_TEXT,"-");    
    }
  //if data exists 
 
  }
//loop to rows 
}
//update table ends here 
//settings structure 
struct impact_colors
{
color insignificant,low,medium,high,none;
impact_colors(void){insignificant=clrGray;low=clrGreen;medium=clrOrange;high=clrRed;none=clrWhite;}
};
impact_colors IC;
void setup_color_grades()
{
 IC.insignificant=T_ColorGrade_Insignificant;
  IC.low=T_ColorGrade_Low;
  IC.medium=T_ColorGrade_Medium;
  IC.high=T_ColorGrade_High;
  IC.none=T_ColorGrade_None;
}
struct ff_settings
{
bool show_insignificant,show_low_impact,show_medium_impact,show_high_impact;
bool show_only_relative;
ff_settings(void){reset();}
void reset(){show_insignificant=true;show_low_impact=true;show_medium_impact=true;show_high_impact=true;show_only_relative=false;}
};
ff_settings FFS;
void load_ffs(string folder,string subfolder,long cid,ff_settings &thesettings)
{
string filename=folder+"\\"+subfolder+"\\"+IntegerToString(cid)+"_ffs.txt";
if(FileIsExist(filename)){
int fop=FileOpen(filename,FILE_READ|FILE_BIN);
if(fop!=INVALID_HANDLE){
FileReadStruct(fop,thesettings);
FileClose(fop);
}}}
void save_ffs(string folder,string subfolder,long cid,ff_settings &thesettings)
{
string filename=folder+"\\"+subfolder+"\\"+IntegerToString(cid)+"_ffs.txt";
if(FileIsExist(filename)){FileDelete(filename);}
int fop=FileOpen(filename,FILE_WRITE|FILE_BIN);
if(fop!=INVALID_HANDLE){
FileWriteStruct(fop,thesettings);
FileClose(fop);
}}
//build settings table 
void build_settings_table(int px,
                          int py,
                          int rowheight,
                          string system_tag,
                          color ss_back_color,color ss_border_color,
                          color ss_row_a_color,
                          color ss_row_b_color,
                          color ss_text_color,
                          color ss_check_mark_color,
                          color ss_check_mark_background_color,
                          impact_colors &the_ic,
                          ff_settings &the_settings){

                          ObjectsDeleteAll(ChartID(),system_tag+"_SET_");

  int drawx=px;
  int drawy=py;
  int windowsizex=rowheight*2+10;
  int windowsizey=(int)(rowheight*7.5)+10;
  int rowsizex=0;
  string texts[]={"All","Pair"};
  //test font size to get windowsize
  TextSetFont(T_TableFont,T_TableFontSize,0,0);
  int max_text_x=0;
  uint w,h,wb,hb;
  TextGetSize(texts[0],w,h);
  TextGetSize(texts[1],wb,hb);
  max_text_x=(int)w*2;
  if(wb>w){max_text_x=(int)wb*2;}
  windowsizex=max_text_x+rowheight+10;
  rowsizex=max_text_x+rowheight;
  
  //background 
  string objna=system_tag+"_SET_SETTINGS_BACK";
  HS_Create_Btn(ChartID(),0,objna,windowsizex,windowsizey,drawx,drawy,T_TableFont,T_TableFontSize,ss_back_color,ss_border_color,BORDER_FLAT,ss_back_color,ALIGN_CENTER,"",false,false);
  //row alternator
  int row_alternator=0;
  color row_color=ss_row_a_color;
  row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
  //impacts 
  drawy+=5;
  drawx+=5;
  //ins
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW1";
    HS_Create_Btn(ChartID(),0,objna,rowsizex,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,row_color,ALIGN_CENTER,"",false,false);
    //display color 
    objna=SystemTag+"_SET_COLOR_DISPLAY_1";
    drawx=5+px+(max_text_x-rowheight)/2;
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,the_ic.insignificant,the_ic.insignificant,BORDER_FLAT,the_ic.insignificant,ALIGN_CENTER,"",false,false);
    objna=SystemTag+"_SET_SELECTOR_INSIGNIFICANT";
    drawx=5+px+max_text_x;
    string ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false);
    
  //low
    row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
    drawy+=rowheight;
    drawx=px+5;
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW2";
    HS_Create_Btn(ChartID(),0,objna,rowsizex,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,row_color,ALIGN_CENTER,"",false,false);
    //display color 
    objna=SystemTag+"_SET_COLOR_DISPLAY_2";
    drawx=5+px+(max_text_x-rowheight)/2;
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,the_ic.low,the_ic.low,BORDER_FLAT,the_ic.low,ALIGN_CENTER,"",false,false);
    objna=SystemTag+"_SET_SELECTOR_LOW";
    drawx=5+px+max_text_x;
    ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false);
  //med
    row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
    drawy+=rowheight;
    drawx=px+5;
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW3";
    HS_Create_Btn(ChartID(),0,objna,rowsizex,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,row_color,ALIGN_CENTER,"",false,false);
    //display color 
    objna=SystemTag+"_SET_COLOR_DISPLAY_3";
    drawx=5+px+(max_text_x-rowheight)/2;
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,the_ic.medium,the_ic.medium,BORDER_FLAT,the_ic.medium,ALIGN_CENTER,"",false,false);
    objna=SystemTag+"_SET_SELECTOR_MEDIUM";
    drawx=5+px+max_text_x;
    ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false); 
  //hig
    row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
    drawy+=rowheight;
    drawx=px+5;
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW4";
    HS_Create_Btn(ChartID(),0,objna,rowsizex,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,row_color,ALIGN_CENTER,"",false,false);
    //display color 
    objna=SystemTag+"_SET_COLOR_DISPLAY_4";
    drawx=5+px+(max_text_x-rowheight)/2;
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,the_ic.high,the_ic.high,BORDER_FLAT,the_ic.high,ALIGN_CENTER,"",false,false);
    objna=SystemTag+"_SET_SELECTOR_HIGH";
    drawx=5+px+max_text_x;
    ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false); 
  //All
    row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
    drawy+=rowheight;
    drawx=px+5;
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW5";
    HS_Create_Btn(ChartID(),0,objna,rowsizex-rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,ss_text_color,ALIGN_CENTER,"All",false,false);
    objna=SystemTag+"_SET_SELECTOR_ALL";
    drawx=5+px+max_text_x;
    ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false); 
    
  //Pair  
    row_alternator++;if(row_alternator>1){row_alternator=0;}if(row_alternator==0){row_color=ss_row_a_color;}else{row_color=ss_row_b_color;}
    drawy+=rowheight;
    drawx=px+5;
    //row background
    objna=system_tag+"_SET_SETTINGS_ROW6";
    HS_Create_Btn(ChartID(),0,objna,rowsizex-rowheight,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,row_color,row_color,BORDER_FLAT,ss_text_color,ALIGN_CENTER,"Pair",false,false);
    objna=SystemTag+"_SET_SELECTOR_PAIR";
    drawx=5+px+max_text_x;
    ct="X";
    HS_Create_Btn(ChartID(),0,objna,rowheight,rowheight,drawx,drawy,"Webdings",T_TableFontSize,ss_check_mark_background_color,ss_check_mark_color,BORDER_FLAT,ss_check_mark_color,ALIGN_CENTER,ct,false,false); 
  //Okay
    drawy+=(int)(rowheight*1.5);
    drawx=px+5;
    objna=system_tag+"_SET_OKAY";
    HS_Create_Btn(ChartID(),0,objna,rowsizex,rowheight,drawx,drawy,T_TableFont,T_TableFontSize,T_ButtonBackColor,T_ButtonBorderColor,BORDER_FLAT,T_ButtonTextColor,ALIGN_CENTER,"Ok",false,false);

    update_settings(system_tag,the_settings);  
    SettingsUp=true;
}
void update_settings(string system_tag,ff_settings &the_settings){
string ct="";
if(the_settings.show_insignificant){ct="a";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_INSIGNIFICANT",OBJPROP_TEXT,ct);
ct="";
if(the_settings.show_low_impact){ct="a";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_LOW",OBJPROP_TEXT,ct);
ct="";
if(the_settings.show_medium_impact){ct="a";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_MEDIUM",OBJPROP_TEXT,ct);
ct="";
if(the_settings.show_high_impact){ct="a";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_HIGH",OBJPROP_TEXT,ct);
ct="a";
if(the_settings.show_only_relative){ct="";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_ALL",OBJPROP_TEXT,ct);
ct="";
if(the_settings.show_only_relative){ct="a";}
ObjectSetString(ChartID(),system_tag+"_SET_SELECTOR_PAIR",OBJPROP_TEXT,ct);
}

//CREATE BTN OBJECT
  void HS_Create_Btn(long cid,
                     int subw,
                     string name,
                     int sx,
                     int sy,
                     int px,
                     int py,
                     string font,
                     int fontsize,
                     color bck_col,
                     color brd_col,
                     ENUM_BORDER_TYPE brd_type,
                     color txt_col,
                     ENUM_ALIGN_MODE align,
                     string text,
                     bool selectable,
                     bool back)  
  {
  bool obji=ObjectCreate(cid,name,OBJ_BUTTON,subw,0,0);
  if(obji)
    {
    ObjectSetString(0,name,OBJPROP_FONT,font);
    ObjectSetInteger(0,name,OBJPROP_ALIGN,align);
    ObjectSetInteger(0,name,OBJPROP_FONTSIZE,fontsize);
    ObjectSetInteger(0,name,OBJPROP_XSIZE,sx);
    ObjectSetInteger(0,name,OBJPROP_YSIZE,sy);
    ObjectSetInteger(0,name,OBJPROP_XDISTANCE,px);
    ObjectSetInteger(0,name,OBJPROP_YDISTANCE,py);
    ObjectSetInteger(0,name,OBJPROP_BGCOLOR,bck_col);
    ObjectSetInteger(0,name,OBJPROP_BORDER_COLOR,brd_col);
    ObjectSetInteger(0,name,OBJPROP_COLOR,txt_col);
    ObjectSetInteger(0,name,OBJPROP_BORDER_TYPE,brd_type);
    ObjectSetInteger(0,name,OBJPROP_SELECTABLE,selectable);
    ObjectSetInteger(0,name,OBJPROP_BACK,back);
    ObjectSetString(0,name,OBJPROP_TEXT,text);
    }
  }                   
//CREATE BTN OBJECT ENDS HERE   
//CREATE INPUT OBJECT
  void HS_Create_Edit(long cid,
                     int subw,
                     string name,
                     int sx,
                     int sy,
                     int px,
                     int py,
                     string font,
                     int fontsize,
                     color bck_col,
                     color brd_col,
                     ENUM_BORDER_TYPE brd_type,
                     color txt_col,
                     ENUM_ALIGN_MODE align,
                     string text,
                     bool selectable,
                     bool readonly,
                     bool back)  
  {
  bool obji=ObjectCreate(cid,name,OBJ_EDIT,subw,0,0);
  if(obji)
    {
    ObjectSetString(0,name,OBJPROP_FONT,font);
    ObjectSetInteger(0,name,OBJPROP_ALIGN,align);
    ObjectSetInteger(0,name,OBJPROP_FONTSIZE,fontsize);
    ObjectSetInteger(0,name,OBJPROP_XSIZE,sx);
    ObjectSetInteger(0,name,OBJPROP_YSIZE,sy);
    ObjectSetInteger(0,name,OBJPROP_XDISTANCE,px);
    ObjectSetInteger(0,name,OBJPROP_YDISTANCE,py);
    ObjectSetInteger(0,name,OBJPROP_BGCOLOR,bck_col);
    ObjectSetInteger(0,name,OBJPROP_BORDER_COLOR,brd_col);
    ObjectSetInteger(0,name,OBJPROP_COLOR,txt_col);
    ObjectSetInteger(0,name,OBJPROP_BORDER_TYPE,brd_type);
    ObjectSetInteger(0,name,OBJPROP_SELECTABLE,selectable);
    ObjectSetInteger(0,name,OBJPROP_READONLY,readonly);
    ObjectSetInteger(0,name,OBJPROP_BACK,back);
    ObjectSetString(0,name,OBJPROP_TEXT,text);
    }
  }                   
//CREATE BTN OBJECT ENDS HERE 

//Load navigation data 
void load_navigation_data(string folder,string subfolder,long for_chart,nav_data &nav){
string filename=folder+"\\"+subfolder+"\\"+IntegerToString(for_chart)+"_nav.txt";
if(FileIsExist(filename)){
int fop=FileOpen(filename,FILE_READ|FILE_BIN);
if(fop!=INVALID_HANDLE){
FileReadStruct(fop,nav);
nav.alert_now=TimeLocal();
nav.alert_prev=(datetime)(nav.alert_now-86400);
nav.alert_next=(datetime)(nav.alert_now+86400);
FileClose(fop);
}
}else{//if file does not exist grab this day
nav.display_now=TimeLocal();
nav.display_prev=(datetime)(nav.display_now-86400);
nav.display_next=(datetime)(nav.display_now+86400);
nav.alert_now=nav.display_now;
nav.alert_next=nav.display_next;
nav.alert_prev=nav.display_prev;
}
}

void save_navigation_data(string folder,string subfolder,long for_chart,nav_data &nav){
string filename=folder+"\\"+subfolder+"\\"+IntegerToString(for_chart)+"_nav.txt";
if(FileIsExist(filename)){FileDelete(filename);}
int fop=FileOpen(filename,FILE_WRITE|FILE_BIN);
if(fop!=INVALID_HANDLE){
FileWriteStruct(fop,nav);
FileClose(fop);
}}


//structure for forex factory news
  struct ff_news
  {
  string title,currency,actual,forecast,previous,timestring,request_param_to_get_this;
  int    impact;//-1 unknown ,0 holiday 1 low 2 medi 3 high
  datetime timelocal,timeutc,datetime_requested;
  bool valid_time,with_data,prints,expecting;
  ff_news(void){datetime_requested=0;expecting=false;prints=false;with_data=false;request_param_to_get_this=NULL;valid_time=false;impact=-1;title=NULL;currency=NULL;actual=NULL;forecast=NULL;previous=NULL;timestring=NULL;timelocal=0;timeutc=0;}
  void add_from_other(ff_news &other){this=other;}
  void setup(string new_title,
             string new_currency,
             string new_actual,
             string new_forecast,
             string new_previous,
             string new_timestring,
             int new_impact,
             int offset_in_minutes_of_request_of_meta_trader,
             string new_request_param_for_this,
             datetime start_of_daystamp,
             datetime new_datetime_requested){
  with_data=true;
  title=new_title;
  currency=new_currency;
  actual=new_actual;
  forecast=new_forecast;
  previous=new_previous;
  prints=false;
  expecting=false;
  if((forecast!="x"&&StringLen(forecast)>0)||(previous!="x"&&StringLen(previous)>0)){prints=true;
  if(actual=="x"){expecting=true;}
  }
  timestring=new_timestring;
  impact=new_impact;
  valid_time=false;
  request_param_to_get_this=new_request_param_for_this;
  datetime_requested=new_datetime_requested;
  //valid time starts here
  if(StringFind(timestring,"ative",0)==-1&&StringFind(timestring,"All Day",0)==-1&&StringFind(timestring,"-",0)==-1){
  valid_time=true;
  //turn into time 
       
       //timelocal=(datetime)(TimeGMT()+(int)(offset_in_minutes_of_request_of_meta_trader*60));
       timelocal=start_of_daystamp;
  bool is_am=false,is_pm=false;
       string timecalcs=timestring;
       bool tl=StringToLower(timecalcs);
       int rep=StringReplace(timecalcs,"am","");
       if(rep>0){is_am=true;}else{rep=StringReplace(timecalcs,"pm","");is_pm=true;}
  ushort usep=StringGetCharacter(":",0);
  string time_parts[];
  int k=StringSplit(timecalcs,usep,time_parts);
  if(k==2){
  int h=(int)StringToInteger(time_parts[0]);
  int m=(int)StringToInteger(time_parts[1]);
  if(is_pm){h+=12;}
  timelocal=(datetime)((int)timelocal+h*3600+m*60);
  //back to gmt 
  timelocal=(datetime)((int)timelocal-(int)(offset_in_minutes_of_request_of_meta_trader*60));
  //back to this 
  timelocal=(datetime)((int)timelocal-(int)TimeGMTOffset());
  }
  }
  //valid time ends here 
  }
  void write_to_file(int fop){
  title=TextGapsTreat(title);
  currency=TextGapsTreat(currency);
  actual=TextGapsTreat(actual);
  forecast=TextGapsTreat(forecast);
  previous=TextGapsTreat(previous);
  timestring=TextGapsTreat(timestring);
  request_param_to_get_this=TextGapsTreat(request_param_to_get_this);
  FileWriteString(fop,title+"\n");
  FileWriteString(fop,currency+"\n");
  FileWriteString(fop,actual+"\n");
  FileWriteString(fop,forecast+"\n");
  FileWriteString(fop,previous+"\n");
  FileWriteString(fop,timestring+"\n");
  FileWriteString(fop,request_param_to_get_this+"\n");
  FileWriteString(fop,IntegerToString(impact)+"\n");
  FileWriteString(fop,TimeToString(timelocal,TIME_DATE|TIME_MINUTES|TIME_SECONDS)+"\n");
  FileWriteString(fop,TimeToString(timeutc,TIME_DATE|TIME_MINUTES|TIME_SECONDS)+"\n");
  FileWriteString(fop,IntegerToString(valid_time)+"\n");
  FileWriteString(fop,IntegerToString(with_data)+"\n");  
  FileWriteString(fop,TimeToString(datetime_requested,TIME_DATE|TIME_MINUTES|TIME_SECONDS)+"\n");
  }
  void read_from_file(int fop){
  title=FileReadString(fop);
  currency=FileReadString(fop);
  actual=FileReadString(fop);
  forecast=FileReadString(fop);
  previous=FileReadString(fop);
  timestring=FileReadString(fop);
  request_param_to_get_this=FileReadString(fop);
  impact=(int)StringToInteger(FileReadString(fop));
  timelocal=(datetime)StringToTime(FileReadString(fop));
  timeutc=(datetime)StringToTime(FileReadString(fop));
  valid_time=(bool)StringToInteger(FileReadString(fop));
  with_data=(bool)StringToInteger(FileReadString(fop));
  datetime_requested=(datetime)(StringToTime(FileReadString(fop)));
  prints=false;
  expecting=false;
  if((forecast!="x"&&StringLen(forecast)>0)||(previous!="x"&&StringLen(previous)>0)){prints=true;
  if(actual=="x"){expecting=true;}
  }
  }
  };

//get specific date data from forex factory calendar
simple_result ff_result;
int getDayFromForexFactory(datetime day,
                           ff_news &receiver[],
                           bool auto_time_zone,
                           int offset_in_minutes_manual,
                           uint request_timeout,
                           bool saved_version_acceptable,
                           bool save_cache){
int res=-1; 
    int offset_in_minutes_used=offset_in_minutes_manual;
    //format date request 
    /* zample : https://www.forexfactory.com/calendar?day=aug11.2021 */
    string param=date_to_ff_date_param(day);
    //daystamp ,timestamp for the start of day ,this is irrelevant to timezone 
    datetime start_of_daystamp=(datetime)(((int)day)-TimeHour(day)*3600-TimeMinute(day)*60-TimeSeconds(day)+1);

string cachename=param;int frep=StringReplace(cachename,".","_");
cachename=SystemFolder+"\\"+CacheFolder+"\\"+cachename+".txt";
//if data exists and accepted 
if(FileIsExist(cachename)&&saved_version_acceptable){
int fop=FileOpen(cachename,FILE_READ|FILE_TXT);
if(fop!=INVALID_HANDLE){
res=200;
//read #
int tots=(int)StringToInteger(FileReadString(fop));
//if any
if(tots>0){
ArrayResize(receiver,tots,0);
for(int l=0;l<tots;l++){
receiver[l].read_from_file(fop);
}
}
FileClose(fop);
}
}else {//if data does not exist or we dont accept it 
    web_request FF;
    string urli="https://www.forexfactory.com/calendar?day="+UrlEncode(param);
    FF.SetupWithBasicHeaders(wr_type_get,urli,request_timeout);
    FF.RequestHeaders["Accept-language"]="en";
    //FF.RequestHeaders["X-Timezone-Offset"]=IntegerToString(((int)(TimeGMTOffset()/-3600)));
    FF.SetReferer("https://www.google.com/search?forexcalendar");//
    FF.SetUserAgent("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:90.0) Gecko/20100101 Firefox/90.0");//firefox user agent 
    //request 
    ff_result=FF.Send();
    res=ff_result.status;
    //if all okay 
    if(res==200){
    //if autotimezone
      if(auto_time_zone){
      string t_find="timezone:";
      int t_found=StringFind(ff_result.response_body,t_find,0);
      if(t_found!=-1){
         int t_found_b=StringFind(ff_result.response_body,".",t_found);
         if(t_found_b!=-1){
           string hourz=StringSubstr(ff_result.response_body,t_found+StringLen(t_find),t_found_b-(t_found+StringLen(t_find)));
           //sign
           t_find=".replace('";
           t_found=StringFind(ff_result.response_body,t_find,0);
           if(t_found!=-1){
           string hourz_sign=StringSubstr(ff_result.response_body,t_found+StringLen(t_find),1);
           if(hourz_sign=="+"){
           int r=StringReplace(hourz,"'","");
           int hh=(int)StringToInteger(hourz);
           offset_in_minutes_used=hh*60;
           //Alert("Offset Found : "+offset_in_minutes_used+" / "+ManualTimeZoneMinutesOffset);
           }
           else if(hourz_sign=="-"){
           int r=StringReplace(hourz,"'","");
           int hh=(int)StringToInteger(hourz);
           offset_in_minutes_used=hh*(-60);
           //Alert("Offset Found : "+offset_in_minutes_used+" / "+ManualTimeZoneMinutesOffset);
           } 
      }}}}
    //if autotimezone ends here 
    string divisor_of_sections="<td class=\"calendar__cell calendar__impact impact calendar__impact calendar__impact";
    string full_data=ff_result.response_body;
    int replace=StringReplace(full_data,"!","");
        replace=StringReplace(full_data,divisor_of_sections,"!");
        //
        //Alert(result.response_body);
    //split by ! 
    string parts[];ushort usep=StringGetCharacter("!",0);
    int total_parts=StringSplit(full_data,usep,parts);
    ArrayFree(receiver);
    //auto timezone on
    //loop into parts
    string previous_timestring="";
      for(int p=1;p<total_parts;p++)
      {
      //get currency from previous part ! 
        string currency=NULL;
        string c_tofind="calendar__currency currency \" title=\"\">";
        int c_found=StringFind(parts[p-1],c_tofind,0);
        if(c_found!=-1){
        int c_found_b=StringFind(parts[p-1],"</td>",c_found);
        if(c_found_b!=-1){
        currency=StringSubstr(parts[p-1],(c_found+StringLen(c_tofind)),(c_found_b-(c_found+StringLen(c_tofind))));
        }}
      //get impact --low --medium --high --holiday
        int impact=-1;
        
             if(StringFind(parts[p],"-holiday\">",0)!=-1){impact=0;}
        else if(StringFind(parts[p],"-low\">",0)!=-1){impact=1;}
        else if(StringFind(parts[p],"-medium\">",0)!=-1){impact=2;}
        else if(StringFind(parts[p],"-high\">",0)!=-1){impact=3;}
      //title 
        string tofind="class=\"calendar__event-title\">";
        int found=StringFind(parts[p],tofind,0);
        if(found!=-1){
        int found_b=StringFind(parts[p],"</span>",found);
        if(found_b!=-1){
        int letitlesize=found_b-(found+StringLen(tofind));
        string title="";
        if(letitlesize>0&&letitlesize<500){title=StringSubstr(parts[p],found+StringLen(tofind),(found_b-(found+StringLen(tofind))));
        //find actual entry 
          tofind="calendar__actual actual\">";
          found=StringFind(parts[p],tofind,found_b);
          if(found!=-1){
          found_b=StringFind(parts[p],"</td>",found);
          if(found_b!=-1){
          int size=found_b-(found+StringLen(tofind));
          string actual_value="x";
          if(size>0){actual_value=StringSubstr(parts[p],found+StringLen(tofind),(found_b-(found+StringLen(tofind))));}
          //find forecast entry 
          tofind="calendar__forecast forecast\">";
          found=StringFind(parts[p],tofind,found_b);
          if(found!=-1){
          found_b=StringFind(parts[p],"</td>",found);
          if(found_b!=-1){
          size=found_b-(found+StringLen(tofind));
          string forecast_value="x";
          if(size>0){forecast_value=StringSubstr(parts[p],found+StringLen(tofind),(found_b-(found+StringLen(tofind))));}
          //find previous entry 
          tofind="calendar__previous previous\">";
          found=StringFind(parts[p],tofind,found_b);
          if(found!=-1){
          found_b=StringFind(parts[p],"</td>",found);
          if(found_b!=-1){
          size=found_b-(found+StringLen(tofind));
          string previous_value="x";
          if(size>0){previous_value=StringSubstr(parts[p],found+StringLen(tofind),(found_b-(found+StringLen(tofind))));}
          //time is on the previous part as well
          tofind="calendar__time time\">";//HH:MMam/pm
          found=StringFind(parts[p-1],tofind,0);
          if(found!=-1){
          found_b=StringFind(parts[p-1],"</td>",found);
          if(found_b!=-1){
          size=found_b-(found+StringLen(tofind));
          string timestring=previous_timestring;
          if(size>0){timestring=StringSubstr(parts[p-1],found+StringLen(tofind),(found_b-(found+StringLen(tofind))));}
          previous_timestring=timestring;
          actual_value=span_killer(actual_value);
          forecast_value=span_killer(forecast_value);
          previous_value=span_killer(previous_value);
          timestring=a_killer(span_killer(timestring));
          //tally 
            currency=new_gap_blaster(currency,"");
            int newsize=ArraySize(receiver)+1;
            ArrayResize(receiver,newsize,0);
            receiver[newsize-1].setup(title,currency,actual_value,forecast_value,previous_value,timestring,impact,offset_in_minutes_used,param,start_of_daystamp,day);
           
        }}}}}}}}}}}
      //title ends here 
      }
    //loop into parts 
    //if save allowed
      if(save_cache){
      int fop=FileOpen(cachename,FILE_WRITE|FILE_TXT);
      if(fop!=INVALID_HANDLE){
      FileWriteString(fop,IntegerToString(ArraySize(receiver))+"\n");
      //loop and write 
        for(int l=0;l<ArraySize(receiver);l++)
        {
        receiver[l].write_to_file(fop);
        }
      //loop and write 
      FileClose(fop);
      }}
    //if save allowed ends here 
    }else{Print("RES["+IntegerToString(res)+"] "+ff_result.error);}
  }
//if data does not exist or we dont accept it ends here
return(res);
}
//get specific date data from forex factory calendar ends here 
//date to forex factory day url parameter ,format monDD.YYYY ,mon lower caps ,DD can be D YYYY always 4 digits
string date_to_ff_date_param(datetime date){
string result=NULL;
//get month 
string mons[]={"void","jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};
result=mons[((uint)TimeMonth(date))]+IntegerToString(TimeDay(date))+"."+IntegerToString(TimeYear(date));
return(result);
}
string new_gap_blaster(string original,string replace_with_character){
string result=original;
int replace=StringReplace(result,CharToString(9),replace_with_character);
    replace=StringReplace(result,CharToString(32),replace_with_character);
ushort usep=StringGetCharacter(replace_with_character,0);
string parts[];
int total=StringSplit(result,usep,parts);
//loop to total
  result="";
  for(int t=0;t<total;t++)
  {
  if(StringLen(parts[t])>0){
  result+=parts[t]+replace_with_character;
  }
  }
//loop to total ends here 
return(result);
}
string span_killer(string feed){
string result=feed;
int r=StringReplace(result,"</span>","");
while(StringFind(result,"<span",0)!=-1){
int from=StringFind(result,"<span",0);
int to=StringFind(result,">",from+1);
int length=to-(from)+1;
string remov=StringSubstr(result,from,length);
r=StringReplace(result,remov,"");
}
return(result);
}
string a_killer(string feed){
string result=feed;
int r=StringReplace(result,"</a>","");
    r=StringReplace(result,"&nbsp;","");
while(StringFind(result,"<a",0)!=-1){
int from=StringFind(result,"<a",0);
int to=StringFind(result,">",from+1);
int length=to-(from)+1;
string remov=StringSubstr(result,from,length);
r=StringReplace(result,remov,"");
}
return(result);
}
//request one page with the parameter
bool requestOneWithParameter(string param,datetime dayrequested)
{
bool result=false;
ff_news interim[];
int interim_request=getDayFromForexFactory(dayrequested,interim,AutoTimeZone,ManualTimeZoneMinutesOffset,RequestTimeout,false,true);
if(interim_request==200){result=true;
//pass all and match to Monitor
  for(int m=0;m<ArraySize(MonitorNews);m++)
  {
  if(MonitorNews[m].title==interim[m].title&&MonitorNews[m].impact==interim[m].impact){
  MonitorNews[m].add_from_other(interim[m]);
  }
  }
}
return(result);
}
//get triple day for filling in todays events locally
bool getTripleData(datetime day,
                   ff_news &receiver[],
                   bool auto_time_zone,
                   int offset_in_minutes_manual,
                   uint request_timeout,
                   bool saved_data_acceptable,
                   bool allow_save){
bool result=false;
ff_news receiver_a[],receiver_b[],receiver_c[];
//get days 
  datetime day1=(datetime)((int)day-86400);
  datetime day2=(datetime)day;
  datetime day3=(datetime)((int)day+86400);
  //day 1
  int request_1=getDayFromForexFactory(day1,receiver_a,auto_time_zone,offset_in_minutes_manual,request_timeout,saved_data_acceptable,allow_save);
  //if it succeeds
  if(request_1==200){
    //Sleep(1000);
    int request_2=getDayFromForexFactory(day2,receiver_b,auto_time_zone,offset_in_minutes_manual,request_timeout,saved_data_acceptable,allow_save);
    //if it succeeds
      if(request_2==200){
        //Sleep(1000);
        int request_3=getDayFromForexFactory(day3,receiver_c,auto_time_zone,offset_in_minutes_manual,request_timeout,saved_data_acceptable,allow_save);
        //if it succeeds 
        if(request_3==200){
        Print("Scanned succesful");
        //crunch first day 
          for(int c=0;c<ArraySize(receiver_a);c++)
          {
          if(receiver_a[c].with_data){
          //if the time is inside our day 
            if(receiver_a[c].valid_time&&TimeDay(receiver_a[c].timelocal)==TimeDay(day)){
            int newsize=ArraySize(receiver)+1;
            ArrayResize(receiver,newsize,0);
            receiver[newsize-1].add_from_other(receiver_a[c]);
            }}
          }
        //crunch second day ,we also get ,tentatives ,alldays and holidays from here  
          for(int c=0;c<ArraySize(receiver_b);c++)
          {
          if(receiver_b[c].with_data){
          //if the time is inside our day 
            if(!receiver_b[c].valid_time||(receiver_b[c].valid_time&&TimeDay(receiver_b[c].timelocal)==TimeDay(day))){
            int newsize=ArraySize(receiver)+1;
            ArrayResize(receiver,newsize,0);
            receiver[newsize-1].add_from_other(receiver_b[c]);
            }}
          }      
        //crunch third day 
          for(int c=0;c<ArraySize(receiver_c);c++)
          {
          if(receiver_c[c].with_data){
          //if the time is inside our day 
            if(receiver_c[c].valid_time&&TimeDay(receiver_c[c].timelocal)==TimeDay(day)){
            int newsize=ArraySize(receiver)+1;
            ArrayResize(receiver,newsize,0);
            receiver[newsize-1].add_from_other(receiver_c[c]);
            }}
          }  
        result=true;
        }
      }else{Print("Cannot Scan Day2 ["+IntegerToString(request_2)+"] error "+IntegerToString(GetLastError()));}
  }else{Print("Cannot Scan Day1 ["+IntegerToString(request_1)+"] error "+IntegerToString(GetLastError()));}
return(result);
}

ff_news DisplayNews[],MonitorNews[],FilteredNews[];

void NewsFiltration(ff_news &filtered[],ff_news &source[],ff_settings &settings){
//empty filtered 
  ArrayFree(filtered);
  string pair1=SymbolInfoString(_Symbol,SYMBOL_CURRENCY_MARGIN);
  string pair2=SymbolInfoString(_Symbol,SYMBOL_CURRENCY_PROFIT);
  
  for(int t=0;t<ArraySize(source);t++)
  {
  bool addit=false;
  if(source[t].with_data){
  //impact filters 
    if((source[t].impact==0&&settings.show_insignificant)||(source[t].impact==1&&settings.show_low_impact)||(source[t].impact==2&&settings.show_medium_impact)||(source[t].impact==3&&settings.show_high_impact)){
    //pair 
      if(settings.show_only_relative){
      if(pair1!=pair2)
        {
        string le_p1=source[t].currency;
        if(StringFind(le_p1,pair1,0)!=-1||StringFind(le_p1,pair2,0)!=-1){
        addit=true;
        }
        }
      }else if(settings.show_only_relative==false){
      addit=true;
      }
    if(addit){
    int newsize=ArraySize(filtered)+1;
    ArrayResize(filtered,newsize,0);
    filtered[newsize-1].add_from_other(source[t]);
    }
    }
  //impact filters 
  }}

}

//inform of the upcoming events 
void inform_upcoming(ff_news &the_event)
{
string title="Upcoming Event Alert";
string message=the_event.title+" ("+TextGapsTreat(the_event.currency)+") in "+IntegerToString(TimeForAlert)+" minutes";
if(alertPopup){Alert(message);}
if(alertPush){SendNotification(message);}
if(alertMail){SendMail(title,message);}
}